/* -*- Javascript -*- */

/* global variables */

var file = null;
var nb_items = 0;
var config = new Array ();
var export_filename = 'snmp.tree';
var subtree_name = '';

/* object type*/

function show_hide_proc (ref, way) {

  if (!ref)
    return false;
  if (typeof ref.leaf !== 'undefined')
    return false;

  if (way != 0) {
    var ul = ref.parentNode.lastChild;
    for (var i in ul.childNodes)
      if (typeof ul.childNodes[i].childNodes !== 'undefined')
        show_hide_proc (ul.childNodes[i].childNodes[0] , way);
  }

  if (typeof ref.style.foregroundColor === 'undefined')
    ref.style.foregroundColor = 'red';
  if (((way == -1) && (ref.style.foregroundColor == 'red')) ||
      ((way ==  1) && (ref.style.foregroundColor != 'red')) ||
      (way  ==  0)) {
    var color = ref.style.color;
    ref.style.color = ref.style.foregroundColor;
    ref.style.foregroundColor = color;
  }

  ref = ref.parentNode.lastChild;

  if (typeof ref.style.displayStyle === 'undefined')
    ref.style.displayStyle = 'none';
  if (typeof ref.style.display === 'undefined')
    ref.style.display = 'block';

  if (((way == -1) && (ref.style.displayStyle == 'none')) ||
      ((way ==  1) && (ref.style.displayStyle != 'none')) ||
      (way  ==  0)) {
    var display = ref.style.display;
    ref.style.display = ref.style.displayStyle;
    ref.style.displayStyle = display;
  }

  return false;
}

function show_hide (event) {

  event = event || window.event;
  var ref = event.target || event.srcElement;

  return show_hide_proc (ref , 0);
}

function trim (str) {
  return str.replace (/^  */, '').replace (/  *$/, '');
}

function compare (base, str) {
  var substr = str.substring (0, base.length);
  return (base == substr);
}

/* import procedure */

function import_proc (data) {

  update_status ('start refreshing', 0);

  /* look for previous tree */
  var tree = document.getElementById ('tree');
  if (!tree) {
    update_status ('can\'t find previous tree');
    return;
  }
  var father = tree.parentNode;

  /* save input data */
  file = data;

  if (subtree_name == '') {
    /* erase old config */
    config = new Array ();
    nb_items = 0;

    /* remove old tree */
    father.removeChild (tree);

    /* create new tree */
    tree = document.createElement ('ul');
    tree.id = 'tree';
    tree.className = 'tree';
    father.appendChild (tree);

    /* add new table into html page */
    father.appendChild (tree);

  } else {

    var ref = document.getElementById (subtree_name);
    if (ref) {
      tree = ref;

      /* remove old subtree */
      function recursive_delete (_ref) {
        if ((typeof _ref.childElementCount === 'undefined') || (_ref.childElementCount == 0))
          return;
        for (var i = _ref.childElementCount - 1; i >= 0; i--) {
          var child = _ref.childNodes[i];
          recursive_delete (child);
          _ref.removeChild (child);
        }
      }
      recursive_delete (tree.parentNode.lastChild);
    }
  }

  if (tree == null)
    return;

  /* process line by line */
  var tab = data.split ('\n');
  for (var i in tab) {

    /* get variable name */
    var line = tab[i].split ('=');
    if (line.length < 2) continue;
    var variable = trim (line[0]);

    /* get type name */
    var buffer = line[1].split (':');
    if (buffer.length < 2) continue;
    var type = trim (buffer[0]);

    /* get value */
    var value = buffer[1];
    for (var j = 2; j < buffer.length; j++)
      value += ":" + buffer[j];
    value = trim (value);

    /* get tree structure */
    var names = variable.split ('.');
    var ancestror = tree;
    var name = "tree";
    var p = null;
    for (var j in names) {
      name += "_" + names[j];
      if ((subtree_name) && (!compare (subtree_name + '_', name)))
        continue;
      var element = document.getElementById (name);
      if (!element) {

        /* create new configuation element */
        var id = nb_items++;
        config[id] = name;

        /* grow up a branch */
        var subtree = document.createElement ('li');
        p = document.createElement ('p');
        p.innerHTML = names[j];
        p.id = 'var_' + id;
        subtree.appendChild (p);
        element = document.createElement ('ul');
        element.id = name;
        subtree.appendChild (element);

        /* add event callbacks*/
        add_event (p, 'click', show_hide);
        p.setAttribute ('oncontextmenu', 'javascript: show_menu (event, ' + id + '); return false;');

        /* connect branch into tree */
        ancestror.appendChild (subtree);
      }
      ancestror = element;
    }

    /* grow up a leaf */
    if (p) {
      p.leaf = true;
      var prev = ancestror.parentNode;
      prev.removeChild (ancestror);
      var span = document.createElement ('span');
      span.innerHTML = value;
      span.title = type;
      prev.appendChild (span);
    }
  }

  subtree_name = '';

  show_tab ('browser');
  update_status ('refresh done', 0);
}

function refresh () {

  if (last_action != '')
    return;
  if (!subtree_name)
    show_tab ('help');

  apply_file ();

  while (last_action != '') {
    /* nothing */
  }

  load_file ();

  return false;
}

function export_proc() {
  return file;
}

/* menu definition */

menu = new Array (menuCallback ('Show subtree', null, function (id) { return show_hide_proc (document.getElementById ('var_' + id), 1);}),
                  menuCallback ('Hide subtree', null, function (id) { return show_hide_proc (document.getElementById ('var_' + id), -1);}),
                  menuCallback ('Refresh subtree', null, function (id) {
                                  var ref = document.getElementById ('var_' + id);
                                  subtree_name = ref.parentNode.lastChild.id;
                                  return refresh ();
                                }));
